import requests
from random import randint
from re import findall
# rootVIII
# pycodestyle validated
# 2018-2020


class ProxyRequests:
    def __init__(self, url):
        self.url = url
        self.sockets = []
        self.rdata = {
            'headers': {},
            'json': {},
            'status_code': 0,
            'timeout': 3.0,
            'errs': [
                'ConnectTimeout',
                'ProxyError',
                'SSLError',
                'ReadTimeout',
                'ConnectionError',
                'ConnectTimeoutError'
            ]
        }
        self.empty_warn = 'Proxy Pool has been emptied'
        self._acquire_sockets()

    def _acquire_sockets(self):
        r = requests.get('https://www.sslproxies.org/')
        matches = findall(r"<td>\d+\.\d+\.\d+\.\d+</td><td>\d+</td>", r.text)
        revised = [m.replace('<td>', '') for m in matches]
        self.sockets = [s[:-5].replace('</td>', ':') for s in revised]

    def _set_request_data(self, req, socket):
        self.rdata['request'] = req.text
        self.rdata['headers'] = req.headers
        self.rdata['status_code'] = req.status_code
        self.rdata['url'] = req.url
        self.rdata['raw'] = req.content
        self.rdata['proxy'] = socket
        try:
            self.rdata['json'] = req.json()
        except Exception as err:
            self.rdata['json'] = {type(err).__name__: str(err)}

    def _rand_sock(self):
        return randint(0, len(self.sockets) - 1)

    def _is_err(self, err):
        if type(err).__name__ not in self.rdata['errs']:
            raise err

    def _limit_succeeded(self):
        raise Exception(self.empty_warn)

    def get(self):
        if len(self.sockets) > 0:
            current_socket = self.sockets.pop(self._rand_sock())
            proxies = {
                'http': 'http://' + current_socket,
                'https': 'https://' + current_socket
            }
            try:
                request = requests.get(
                    self.url,
                    timeout=self.rdata['timeout'],
                    proxies=proxies)
                self._set_request_data(request, current_socket)
            except Exception as e:
                self._is_err(e)
                self.get()
        else:
            self._limit_succeeded()

    def get_with_headers(self):
        if len(self.sockets) > 0:
            current_socket = self.sockets.pop(self._rand_sock())
            proxies = {
                'http': 'http://' + current_socket,
                'https': 'https://' + current_socket
            }
            try:
                request = requests.get(
                    self.url,
                    timeout=self.rdata['timeout'],
                    proxies=proxies,
                    headers=self.rdata['headers'])
                self._set_request_data(request, current_socket)
            except Exception as e:
                self._is_err(e)
                self.get_with_headers()
        else:
            self._limit_succeeded()

    def post(self, data):
        if len(self.sockets) > 0:
            current_socket = self.sockets.pop(self._rand_sock())
            proxies = {
                'http': 'http://' + current_socket,
                'https': 'https://' + current_socket
            }
            try:
                request = requests.post(
                    self.url,
                    json=data,
                    timeout=self.rdata['timeout'],
                    proxies=proxies)
                self._set_request_data(request, current_socket)
            except Exception as e:
                self._is_err(e)
                self.post(data)
        else:
            self._limit_succeeded()

    def post_with_headers(self, data):
        if len(self.sockets) > 0:
            current_socket = self.sockets.pop(self._rand_sock())
            proxies = {
                'http': 'http://' + current_socket,
                'https': 'https://' + current_socket
            }
            try:
                request = requests.post(
                    self.url,
                    json=data,
                    timeout=self.rdata['timeout'],
                    headers=self.rdata['headers'],
                    proxies=proxies)
                self._set_request_data(request, current_socket)
            except Exception as e:
                self._is_err(e)
                self.post_with_headers(data)
        else:
            self._limit_succeeded()

    def post_file(self):
        if len(self.sockets) > 0:
            current_socket = self.sockets.pop(self._rand_sock())
            proxies = {
                'http': 'http://' + current_socket,
                'https': 'https://' + current_socket
            }
            try:
                request = requests.post(
                    self.url,
                    proxies=proxies,
                    timeout=self.rdata['timeout'],
                    files={'upload_file': open(self.rdata['file'], 'rb')})
                self._set_request_data(request, current_socket)
            except Exception as e:
                self._is_err(e)
                self.post_file()
        else:
            self._limit_succeeded()

    def post_file_with_headers(self):
        if len(self.sockets) > 0:
            current_socket = self.sockets.pop(self._rand_sock())
            proxies = {
                'http': 'http://' + current_socket,
                'https': 'https://' + current_socket
            }
            try:
                request = requests.post(
                    self.url,
                    files={'upload_file': open(self.rdata['file'], 'rb')},
                    timeout=self.rdata['timeout'],
                    headers=self.rdata['headers'],
                    proxies=proxies)
                self._set_request_data(request, current_socket)
            except Exception as e:
                self._is_err(e)
                self.post_file_with_headers()
        else:
            self._limit_succeeded()

    def get_headers(self):
        return self.rdata['headers']

    def set_headers(self, outgoing_headers):
        self.rdata['headers'] = outgoing_headers

    def set_file(self, outgoing_file):
        self.rdata['file'] = outgoing_file

    def get_status_code(self):
        return self.rdata['status_code']

    def get_proxy_used(self):
        return self.rdata['proxy']

    def get_raw(self):
        return self.rdata['raw']

    def get_json(self):
        return self.rdata['json']

    def get_url(self):
        return self.rdata['url']

    def __str__(self):
        return str(self.rdata['request'])


class ProxyRequestsBasicAuth(ProxyRequests):
    def __init__(self, url, username, password):
        super().__init__(url)
        self.username = username
        self.password = password

    def get(self):
        if len(self.sockets) > 0:
            current_socket = self.sockets.pop(self._rand_sock())
            proxies = {
                'http': 'http://' + current_socket,
                'https': 'https://' + current_socket
            }
            try:
                request = requests.get(
                    self.url,
                    auth=(self.username, self.password),
                    timeout=self.rdata['timeout'],
                    proxies=proxies)
                self._set_request_data(request, current_socket)
            except Exception as e:
                self._is_err(e)
                self.get()
        else:
            self._limit_succeeded()

    def get_with_headers(self):
        if len(self.sockets) > 0:
            current_socket = self.sockets.pop(self._rand_sock())
            proxies = {
                'http': 'http://' + current_socket,
                'https': 'https://' + current_socket
            }
            try:
                request = requests.get(
                    self.url,
                    auth=(self.username, self.password),
                    timeout=self.rdata['timeout'],
                    proxies=proxies,
                    headers=self.rdata['headers'])
                self._set_request_data(request, current_socket)
            except Exception as e:
                self._is_err(e)
                self.get_with_headers()
        else:
            self._limit_succeeded()

    def post(self, data):
        if len(self.sockets) > 0:
            current_socket = self.sockets.pop(self._rand_sock())
            proxies = {
                'http': 'http://' + current_socket,
                'https': 'https://' + current_socket
            }
            try:
                request = requests.post(
                    self.url,
                    json=data,
                    auth=(self.username, self.password),
                    timeout=self.rdata['timeout'],
                    proxies=proxies)
                self._set_request_data(request, current_socket)
            except Exception as e:
                self._is_err(e)
                self.post(data)
        else:
            self._limit_succeeded()

    def post_with_headers(self, data):
        if len(self.sockets) > 0:
            current_socket = self.sockets.pop(self._rand_sock())
            proxies = {
                'http': 'http://' + current_socket,
                'https': 'https://' + current_socket
            }
            try:
                request = requests.post(
                    self.url,
                    json=data,
                    auth=(self.username, self.password),
                    timeout=self.rdata['timeout'],
                    headers=self.rdata['headers'],
                    proxies=proxies)
                self._set_request_data(request, current_socket)
            except Exception as e:
                self._is_err(e)
                self.post_with_headers(data)
        else:
            self._limit_succeeded()

    def post_file(self):
        if len(self.sockets) > 0:
            current_socket = self.sockets.pop(self._rand_sock())
            proxies = {
                'http': 'http://' + current_socket,
                'https': 'https://' + current_socket
            }
            try:
                request = requests.post(
                    self.url,
                    files={'upload_file': open(self.rdata['file'], 'rb')},
                    auth=(self.username, self.password),
                    timeout=self.rdata['timeout'],
                    proxies=proxies)
                self._set_request_data(request, current_socket)
            except Exception as e:
                self._is_err(e)
                self.post_file()
        else:
            self._limit_succeeded()

    def post_file_with_headers(self):
        if len(self.sockets) > 0:
            current_socket = self.sockets.pop(self._rand_sock())
            proxies = {
                'http': 'http://' + current_socket,
                'https': 'https://' + current_socket
            }
            try:
                request = requests.post(
                    self.url,
                    files={'upload_file': open(self.rdata['file'], 'rb')},
                    auth=(self.username, self.password),
                    timeout=self.rdata['timeout'],
                    headers=self.rdata['headers'],
                    proxies=proxies)
                self._set_request_data(request, current_socket)
            except Exception as e:
                self._is_err(e)
                self.post_file_with_headers()
        else:
            self._limit_succeeded()



#!/usr/bin/python3

# KNOWN ISSUES AND BUGS | HELP WELCOME
# When scanning, and the x.y domain being tried is not responding, and the user uses CTRL-C, the system throws an error like "handled exception during gaierror exception"
# Proxy scanning is slow and always returns 0 when using .get_status_code(), or None when using .get()

# TODO
# ====
# Implement proxy use for requests
# Fix 'During handling of the above exception, another exception occurred:' error when using CTRL-C during a scan.

import socket
import argparse
import time
import sys
import tlds
# from proxy_requests import ProxyRequests
from __init__ import __version__
version = __version__

list = 'listOfDomains'
targets=[]# Targets to scan
listOfDomains=[]# TLD list to check
result=[]# (Positive) results
args=''
verbose = False
proxy = False
attempts=0# Number of TLDs attempts to find
pos=[]# list of positive results
lastTry=''# Last tried TLD
domainCategory=0# for -domain-category menu
continueDomain = ''# --domain-continue tld
continueDomainIndice=0#--domain-continue tld indice in tlds.py
outputFile=''
max_scans=-1


def main():
		if len(sys.argv) <= 1:
			print('Use -h for help')
		else:
			setTarget()

			scan()

			printer()

			if args.output:
				output(f'''====================
Target domain: {args.target}
====================
Atempted:----------: {attempts}
Positive results:--: {len(pos)}
Last Attempt: -----: {lastTry}''')
				if len(pos)>=1:
					output(f'''========================
[+] Positive Results [+]
========================
{"".join(pos)}''')
				else:
					print('''========================
[-] No Results Found [-]
========================''')


def output(line):
	original_stdout = sys.stdout
	with open(args.output, 'a') as o:
		sys.stdout = o
		print(line)
		sys.stdout = original_stdout

def printer():
	print(f'''====================
Target domain: {args.target}
====================
Atempted:----------: {attempts}
Positive results:--: {len(pos)}
Last Attempt: -----: {lastTry}''')
	if len(pos)>=1:
		print(f'''========================
[+] Positive Results [+]
========================
{"".join(pos)}''')
	else:
		print('''========================
[-] No Results Found [-]
========================''')

def scan():
	print(f'''  _______ _      _____   _____                     _
 |__   __| |    |  __ \ / ____|                   | |
    | |  | |    | |  | | (___   ___  __ _ _ __ ___| |__   ___ _ __
    | |  | |    | |  | |\___ \ / _ \/ _` | '__/ __| '_ \ / _ \ '__|
    | |  | |____| |__| |____) |  __/ (_| | | | (__| | | |  __/ |
    |_|  |______|_____/|_____/ \___|\__,_|_|  \___|_| |_|\___|_|
    https://www.github.com/plague-anon/TLDSearcher/wiki
========================================
[+] Starting TLDSearcher at {time.strftime('%H:%M:%S')} [+]
----------------------------------------
''')# Ascii banner and time started
	if args.output:
		output(f'''  _______ _      _____   _____                     _
 |__   __| |    |  __ \ / ____|                   | |
    | |  | |    | |  | | (___   ___  __ _ _ __ ___| |__   ___ _ __
    | |  | |    | |  | |\___ \ / _ \/ _` | '__/ __| '_ \ / _ \ '__|
    | |  | |____| |__| |____) |  __/ (_| | | | (__| | | |  __/ |
    |_|  |______|_____/|_____/ \___|\__,_|_|  \___|_| |_|\___|_|
    https://www.github.com/plague-anon/TLDSearcher/wiki
========================================
[+] Starting TLDSearcher at {time.strftime('%H:%M:%S')} [+]
----------------------------------------''')# Ascii banner and time started
	for target in targets:
		for tld in listOfDomains:
			url = f'{target}{tld}'
			if verbose:
				print(f'Trying: {url}')

			global lastTry
			lastTry=tld

			global attempts
			attempts += 1

			global max_scans
			if max_scans >=1:
				max_scans -= 1
			elif max_scans == 0:
				sys.exit(printer())

			# if args.proxy:
			# 	proxyScan(url)
			# else:
			# 	normalScan(url, target, tld)
			normalScan(url, target, tld)

# TODO: Too slow and retuns 0
# def proxyScan(url):
# 	try:
# 		r = ProxyRequests(f'https://{url}')
# 		print(r.get_status_code())
# 	except Exception as e:
# 		print(e)


def normalScan(url, target, tld):

	try:
		response = socket.gethostbyname_ex(url)
		if response[2]:
			if verbose:
				print(f'  [+] Found that {target} has TLD of {tld} || hostname: {response[0]} | IP: {response[2]}')
			else:
				print(f'[+] Found that {target} has TLD of {tld}')
			pos.append(f'{target}{tld}\n')
	except socket.gaierror: # No response from server
			if verbose:
				print(f'  [-] No match found for {target}{tld}')

def sortTLD(tld):
# Takes in list of domains from either user -d TLDs or -dF FILE
# If from -d flag, user must use , between tlds they wish to scan
# NOTE: the preceeding . is not needed as this function enters it if missing
	if str(tld).find(','): # tld is from user -d input
		tldsplitlist = str(tld).split(',',-1) # split user input at , (type is list)
		for tld in tldsplitlist:
			if tld[0]!='.':
				listOfDomains.append(f'.{tld}')
			else:
				listOfDomains.append(tld)
	else:
		for t in tld:
			print(t)
			sys.exit()
		# TODO: sanitise input from user specified file

def setVars():
	global listOfDomains
	global outputFile
	global max_scans

	if args.max_scans:
		if int(args.max_scans) >= 1:
			max_scans = int(args.max_scans)-1
		else:
			sys.exit(print('You must give a number >= 1 for --max_scans'))
	#
	# if args.proxy:
	# 	global proxy
	# 	proxy = True

	if args.verbose:
		global verbose
		verbose=True

	if args.domain:
		sortTLD(args.domain)
	elif args.domainFile:
		domainInputFile = open((domainFile), 'r').readlines()
		for d in domainInputFile:
			sortTLD(domainInputFile)
	elif args.domainContinue:
		global continueDomain
		adc = args.domainContinue
		if adc[0] == '.':
			continueDomain = adc
		else:
			continueDomain = f'.{adc}'
		continueDomainIndice = tlds.tldList['all'].index(continueDomain)
		listOfDomains = tlds.tldList['all'][continueDomainIndice:len(tlds.tldList['all'])]
	elif args.domainCategory:
		print('''
[1] Countries ---------	(ccTLD | .ua, .nz, .de, .es, .ru, etc)
[2] Original Generic --	(gTLD |  .com, .net, .org, .info, etc.)
[3] Internet ----------	(.io, .tech, .cloud, .network, etc.)
[4] Money and finance -	(.trade, .marketing, .finance, etc.)
[5] Technology -------- (.tech, .app, .systems, .security, etc.)
[6] Professional ------ (.pro, .work, .consulting, .careers, etc.)
[7] Government -------- (.tax, .army, .gov, .mil, .voting, etc.)
[8] All ---------------	(Scans ALL TLDs)
		''')
		print('Select which category you want to search for.')
		domainChoice = input('Type a number and click ENTER: ')
		listOfDomains = tlds.getTlds(int(domainChoice))
	else:
		listOfDomains = tlds.tldList['all']

	if args.output:
		outputFile = args.output


def setArgs():
	parser = argparse.ArgumentParser(description='Search for active Top Level Domains(TLD\'s) for domain names.',usage='%(prog)s {[-t <TARGET>] [-tF <TARGET_FILE>]} {[-d com,co.uk,.net] [-dF <DOMAIN_FILE>] [-dC] [-dc <DOMAIN_TO_CONTINUE_FROM>]} [-o <OUTPUT_FILE_NAME>] [-v] ')
	parser.add_argument('-t', '--target', help='targets domain name to scan for listOfDomains', action='store')
	parser.add_argument('-tF', '--targetFile', help='Supply a targets file', action='store')
	parser.add_argument('--max_scans', help='Maximum number of TLDs to scan', action='store')
	parser.add_argument('-o', '--output', help='File to output results into', action='store')
	#parser.add_argument('-p', '--proxy', help='Use proxies for requests (VERY SLOW!)', action='store_true')
	parser.add_argument('-v', '--verbose', help='Verbose output mode', action='store_true')
	parser.add_argument('--version', help='Display version information', action='version', version='%(prog)s ' + version)

	group = parser.add_mutually_exclusive_group()
	group.add_argument('-d', '--domain', help='list of domains to scan. (com,ua,nz,de)', action='store')
	group.add_argument('-dC', '--domainCategory', help='Scan specific TLD categories', action='store_true')
	group.add_argument('-dc', '--domainContinue', help='Continue scanning all domains, starting from last attempt "..."', action='store')
	group.add_argument('-dF', '--domainFile', help='List of listOfDomains to scan. (Default = all)', action='store')

	global args
	args = parser.parse_args()

def setTarget():
	global targets
	if args.target:
		targets.append(args.target)
	elif args.targetFile:
		tFile = open(('args.targetFile'), 'r')
		for x in tFile.readlines():
			if checkTarget():
				targets.append(tFile.readlines())

if __name__ == '__main__':
	setArgs()
	setVars()
	main()



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

from tldsearcher.__init__ import __version__

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.md') as history_file:
    history = history_file.read()

requirements = [ ]

setup_requirements = [ ]

test_requirements = [ ]

setup(
    author="plague",
    author_email='plague_anon@protonmail.com',
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="A tool for finding TLDs for domains",
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme + '\n\n' + history,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords='tldsearcher',
    name='tldsearcher',
    packages=find_packages(include=['tldsearcher', 'tldsearcher.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/plague-anon/tldsearcher',
    version=__version__,
    zip_safe=False,
)
